from urllib.parse import urlencode
from jsonfield import JSONField
from django.utils import timezone
from django.utils.translation import gettext as _
from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User
from django.conf import settings
from django.core.exceptions import FieldError
from django.core.exceptions import ValidationError
from . import settings as app_settings


class FilterQuery(models.Model):
    name = models.CharField(max_length=128)
    description = models.TextField(blank=True)
    persistent = models.BooleanField(default=False)
    querydict = JSONField(default=dict())
    created = models.DateTimeField(auto_now_add=True)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    user = models.ForeignKey(User, on_delete=models.CASCADE)

    class Meta:
        ordering = ['-created']

    def save(self, *args, **kwargs):
        # create a generic name if missing
        if not self.name:
            if settings.USE_TZ:
                now = timezone.localtime().strftime('%x %X')
            else:
                now = timezone.now().strftime('%x %X')
            self.name = _('Filter from ') + now

        # create a generic description if missing
        if not self.description:
            self.description = self.pretty_query

        # save filter-query
        super().save(*args, **kwargs)

        # truncate history
        if app_settings.TRUNCATE_HISTORY and not self.persistent:
            history = FilterQuery.objects.filter(
                user=self.user,
                content_type=self.content_type,
                persistent=False
            )[app_settings.HISTORY_LIMIT:]
            FilterQuery.objects.filter(id__in=[f.id for f in history]).delete()

    @property
    def pretty_query(self):
        lines = list()
        for key in sorted(self.querydict.keys()):
            line = '{} = {}'.format(key, self.querydict[key])
            lines.append(line)
        return '\n'.join(lines)

    @property
    def urlquery(self):
        return urlencode(self.querydict)



# Generated by Django 2.2.10 on 2020-05-09 09:23

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FilterQuery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
                ('description', models.TextField(blank=True)),
                ('persistent', models.BooleanField(default=False)),
                ('querydict', jsonfield.fields.JSONField(default={})),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['-created'],
            },
        ),
    ]



from django.conf import settings


HISTORY_LIMIT = getattr(settings, 'ADMIN_FILTER_HISTORY_LIMIT', 3)
TRUNCATE_HISTORY = getattr(settings, 'ADMIN_FILTER_TRUNCATE_HISTORY', True)
URL_PATH = getattr(settings, 'ADMIN_FILTER_URL_PATH', 'filter').strip('/') + '/'



from django.core.exceptions import ImproperlyConfigured
from django_filters.filterset import FilterSetMetaclass
from django_filters.filterset import BaseFilterSet




class AdminFilterSetMetaclass(FilterSetMetaclass):
    """
    Add registry-functionalities to the FilterSetMetaclass.
    """
    def __new__(cls, name, bases, attrs):
        new_class = super().__new__(cls, name, bases, attrs)
        if new_class._meta.model:
            new_class._REGISTRY[new_class._meta.model] = new_class
        return new_class


class AdminFilterSet(BaseFilterSet, metaclass=AdminFilterSetMetaclass):
    _REGISTRY = dict()

    @classmethod
    def by_model(cls, model):
        """
        Return the filterset for a given model.
        """
        try:
            return cls._REGISTRY[model]
        except KeyError:
            msg = "No filterset was declared for model '{}'"
            raise ImproperlyConfigured(msg.format(model.__name__))



# -*- coding: utf-8 -*-
from django.template import Library
from .. import settings


register = Library()


@register.simple_tag
def urlpath():
    return settings.URL_PATH


@register.filter
def persistent(filter):
    return [f for f in filter[1:] if f['filter'].persistent]


@register.filter
def history(filter):
    return [f for f in filter[1:] if not f['filter'].persistent]



import re
from django.utils.translation import gettext as _
from django.conf import settings
from django.contrib import admin

from . import settings as app_settings
from .filterset import AdminFilterSet
from .models import FilterQuery


class CustomFilter(admin.SimpleListFilter):
    title = _('Custom Filters')
    template = 'django_admin_filter/custom_filter.html'
    parameter_name = 'filter_id'

    def __init__(self, request, params, model, model_admin):
        super().__init__(request, params, model, model_admin)
        self.csrftoken = request.META.get('CSRF_COOKIE')
        self.filterset_class = AdminFilterSet.by_model(model)
        try:
            self.current_query = FilterQuery.objects.get(pk=self.value())
        except FilterQuery.DoesNotExist:
            self.current_query = None

    def queryset(self, request, queryset):
        if not self.current_query:
            return queryset

        return self.filterset_class(self.current_query.querydict, queryset).qs

    def has_output(self):
        return True

    def lookups(self, request, model_admin):
        model_name = model_admin.model.__name__.lower()
        params = dict(content_type__model=model_name, user=request.user)
        persistent = FilterQuery.objects.filter(persistent=True, **params)
        history = FilterQuery.objects.filter(persistent=False, **params)
        recent = history[:app_settings.HISTORY_LIMIT]
        return list(persistent) + list(recent)

    def choices(self, changelist):
        if self.lookup_choices:
            remove = self.used_parameters.keys()
            yield {
                'selected': not self.used_parameters,
                'query_string': changelist.get_query_string(remove=remove),
                'display': _('All'),
            }
        for query in self.lookup_choices:
            yield {
                'selected': self.current_query and self.current_query.id == query.id,
                'query_string': changelist.get_query_string(dict(filter_id=query.id)),
                'csrftoken': self.csrftoken,
                'filter': query,
            }



from django import forms
from .models import FilterQuery


class FilterForm(forms.ModelForm):
    name = forms.CharField(
        max_length=128,
        required=False,
        widget=forms.widgets.TextInput(
            attrs=dict(placeholder='optional')
        )
    )
    description = forms.CharField(
        required=False,
        widget=forms.widgets.Textarea(attrs=dict(
            rows=4,
            placeholder='optional'
        ))
    )

    class Meta:
        model = FilterQuery
        fields = ['name', 'description']



#!/usr/bin/env python

import os
from setuptools import setup
from setuptools import find_packages


def read(filename):
    path = os.path.join(os.path.dirname(__file__), filename)
    with open(path, encoding="utf-8") as file:
        return file.read()


version = __import__("django_admin_filter").__version__
if '-dev' in version:
    dev_status = 'Development Status :: 3 - Alpha'
elif '-beta' in version:
    dev_status = 'Development Status :: 4 - Beta'
else:
    dev_status = 'Development Status :: 5 - Production/Stable'


setup(
    name="django-admin-filter",
    version=version,
    description="A generic filter for the django-admin-page based on django-filter.",
    long_description=read("README.rst"),
    author="Thomas Leichtfuß",
    author_email="thomas.leichtfuss@posteo.de",
    url="https://github.com/thomst/django-admin-filter",
    license="BSD License",
    platforms=["OS Independent"],
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    install_requires=[
        "Django>=2.2",
        'django-filter>=2.2',
        'jsonfield>=3.1.0',
    ],
    classifiers=[
        dev_status,
        "Framework :: Django",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
    ],
    zip_safe=True,
)



import html.parser
from decimal import Decimal

from django.http import Http404
from django.http import JsonResponse
from django.shortcuts import render
from django.utils.text import format_lazy
from django.utils.translation import gettext as _
from django.core.exceptions import PermissionDenied
from django.core.exceptions import ImproperlyConfigured
from django.core.exceptions import FieldError
from django.core.exceptions import ValidationError
from django.views.generic.base import TemplateResponseMixin
from django.views.generic.edit import BaseCreateView
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.contenttypes.models import ContentType
from django.urls import reverse
from .filterset import AdminFilterSet
from .models import FilterQuery
from .forms import FilterForm


def permission_required(func):
    """
    Decorator for view-methods to check permission to view the filtered items.
    """
    def wrapper(self, request, **kwargs):
        permission = '{app_label}.view_{model}'.format(**kwargs)
        if not request.user.has_perm(permission):
            raise PermissionDenied
        return func(self, request, **kwargs)
    return wrapper


def setup(func):
    """
    Extract the contenttype from url-paramerters and setup the filter-class.
    """
    def wrapper(self, request, **kwargs):
        try:
            self.content_type = ContentType.objects.get(**kwargs)
        except ContentType.DoesNotExist:
            msg = format_lazy(_("No model '{model}' in app '{app_label}'"), **kwargs)
            raise Http404(msg)
        else:
            ct_model = self.content_type.model_class()
            self.filterset_class = AdminFilterSet.by_model(ct_model)
        return func(self, request, **kwargs)
    return wrapper


class FilterQueryView(LoginRequiredMixin, TemplateResponseMixin, BaseCreateView):
    """
    View to add and delete Filters.
    """
    template_name = 'django_admin_filter/filter_query.html'
    model = FilterQuery
    form_class = FilterForm
    prefix = 'fq'
    object = None

    @permission_required
    def delete(self, *args, **kwargs):
        self.object = self.get_object()
        if not self.object.user == self.request.user:
            raise PermissionDenied
        response = dict(id=self.object.id)
        self.object.delete()
        return JsonResponse(response)

    @setup
    @permission_required
    def get(self, *args, **kwargs):
        return super().get(*args, **kwargs)

    @setup
    @permission_required
    def post(self, *args, **kwargs):
        form = self.get_form()
        query_form = self.get_query_form()

        if form.is_valid() and query_form.is_valid():
            return self.form_valid(form, query_form)
        else:
            return self.form_invalid(form, query_form)

    def get_querydict(self):
        # TODO: Filter out FILTERS_NULL_CHOICE_VALUE for ChoiceFilter and
        # 'unknown' for BooleanFields.
        field_names = self.filterset_class().form.fields.keys()
        fields = self.filterset_class().form.fields
        querydict = dict(
            (k, v) for k, v in self.request.POST.items()
            if k in field_names and not v is str()
        )
        return querydict

    def form_valid(self, form, query_form):
        self.object = form.save(commit=False)
        self.object.querydict = self.get_querydict()
        self.object.content_type = self.content_type
        self.object.persistent = 'save' in self.request.POST
        self.object.user = self.request.user
        self.object.save()
        return super().form_valid(form)

    def form_invalid(self, form, query_form):
        context = self.get_context_data(form=form, query_form=query_form)
        return self.render_to_response(context)

    def get_success_url(self):
        url_pattern = 'admin:{app_label}_{model}_changelist'
        query = '?filter_id={}'.format(self.object.id)
        url = reverse(url_pattern.format(**self.kwargs)) + query
        return url

    def get_query_form(self):
        if self.request.method in ('POST', 'PUT'):
            data = self.request.POST
        else:
            data = dict()
        filter = self.filterset_class(data)
        return filter.form

    def get_context_data(self, **kwargs):
        query_form = self.get_query_form()
        extra_context = dict(query_form=query_form)
        extra_context.update(kwargs)
        return super().get_context_data(**extra_context)



from django.conf.urls import re_path
from .views import FilterQueryView
from . import settings


urlpatterns = [
    re_path(r'^(?P<app_label>\w+)/(?P<model>\w+)/{}$'.format(settings.URL_PATH), FilterQueryView.as_view()),
    re_path(r'^(?P<app_label>\w+)/(?P<model>\w+)/{}(?P<pk>\d+)/$'.format(settings.URL_PATH), FilterQueryView.as_view()),
]
